<?php

namespace Drupal\remote_file_importer_ftp\Plugin\RemoteFileImporter\Datasource;

use Drupal\Core\Form\FormStateInterface;
use Drupal\remote_file_importer\DataSourcePluginBase;

/**
 * Provids FTP import functionality.
 *
 * @RemoteFileImporterDataSourcePlugin(
 *   id = "rif_ftp_datasource",
 *   label = @Translation("FTP"),
 * )
 */
class FtpDataSource extends DataSourcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function getSettingsForm($values = []) {

    $form = [];
    $form['host'] = [
      '#type' => 'textfield',
      '#default_value' => $values['host'] ?? '',
      '#title' => $this->t('Host'),
      '#description' => $this->t('Exclude protocol.'),
      '#required' => TRUE,
    ];

    $form['port'] = [
      '#type' => 'textfield',
      '#default_value' => $values['port'] ?? '',
      '#title' => $this->t('Port'),
      '#placeholder' => '21',
    ];

    $form['ssl'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('SSL / TLS'),
      '#default_value' => $values['ssl'] ?? TRUE,
    ];

    $form['ftp_mode'] = [
      '#type' => 'select',
      '#default_value' => $values['select'] ?? '',
      '#title' => $this->t('Mode'),
      '#options' => [
        'passive' => $this->t('Passive'),
        'active' => $this->t('Active'),
      ],
    ];

    $form['user'] = [
      '#type' => 'textfield',
      '#default_value' => $values['user'] ?? '',
      '#title' => $this->t('User'),
      '#required' => TRUE,
    ];

    $form['password'] = [
      '#type' => 'password',
      '#default_value' => $values['password'] ?? '',
      '#title' => $this->t('Password'),
      '#required' => TRUE,
    ];

    $form['remote_dir'] = [
      '#type' => 'textfield',
      '#default_value' => $values['remote_dir'] ?? '',
      '#title' => $this->t('Remote Dir'),
    ];

    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function validateSettingsForm(array &$form, FormStateInterface $form_state) {
    $input = $form_state->getValue('host');
    if (strpos($input, ':') !== FALSE || strpos($input, '//') !== FALSE) {
      $form_state->setErrorByName('host', $this->t('The host must not contain the protocol (e.g. ftp://).'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function establishConnection($settings) {

    $port = !empty($settings['port']) ? $settings['port'] : 21;

    if ($settings['ssl']) {
      $this->connection = ftp_ssl_connect($settings['host'], $port);
    }
    else {
      $this->connection = ftp_connect($settings['host'], $port);
    }

    if (ftp_login($this->connection, $settings['user'], $settings['password'])) {
      if ($settings['remote_dir']) {
        ftp_chdir($this->connection, $settings['remote_dir']);
      }

      if ($settings['ftp_mode'] == 'passive') {
        ftp_pasv($this->connection, TRUE);
      }

      return TRUE;
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function collectFiles() {
    $remoteDirectory = ftp_pwd($this->connection);
    ftp_pasv($this->connection, TRUE);
    $ftpRawList = ftp_rawlist($this->connection, $remoteDirectory, TRUE);
    $files = [];
    $dir = '';
    foreach ($ftpRawList as $item) {

      if (substr($item, 0, strlen($remoteDirectory)) == $remoteDirectory) {
        $dir = trim(substr($item, strlen($remoteDirectory)), ':');
      }

      if ($item && $item[0] == '-') {
        $itemData = preg_split("/\s+/", $item);
        $file = end($itemData);
        if (count(explode('.', $file)) > 1 && explode('.', $file)[0]) {
          $files[] = [
            'path' => $dir . '/' . $file,
            'metadata' => [
              'modified_date' => ftp_mdtm($this->connection, $remoteDirectory . $dir . '/' . $file),
            ],
          ];
        }
      }
    }

    return $files;

  }

  /**
   * {@inheritdoc}
   */
  public function importFile($localFileName, $remoteFileName) {
    $remoteDirectory = ftp_pwd($this->connection);
    $remoteFilePath = str_replace('//', '/', $remoteDirectory . $remoteFileName);
    ftp_get($this->connection, $localFileName, $remoteFilePath);
  }

  /**
   * {@inheritdoc}
   */
  public function closeConnection() {
    ftp_close($this->connection);
  }

}
